VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CRegistry"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Dichiarazione delle funzioni Api di Windows per il registro
Private Declare Function RegCloseKey Lib "advapi32.dll" (ByVal hKey As Long) As Long
Private Declare Function RegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" (ByVal hKey As Long, ByVal lpSubKey As String) As Long
Private Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal Reserved As Long, ByVal lpClass As String, ByVal dwOptions As Long, ByVal samDesired As Long, lpSecurityAttributes As SECURITY_ATTRIBUTES, phkResult As Long, lpdwDisposition As Long) As Long
Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal ulOptions As Long, ByVal samDesired As Long, phkResult As Long) As Long
Private Declare Function RegSetValueEx Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal Reserved As Long, ByVal dwType As Long, lpData As Any, ByVal cbData As Long) As Long
Private Declare Function RegSetValue Lib "advapi32.dll" Alias "RegSetValueA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal dwType As Long, ByVal lpData As String, ByVal cbData As Long) As Long
Private Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, lpData As Any, lpcbData As Long) As Long
Private Declare Function RegQueryValue Lib "advapi32.dll" Alias "RegQueryValueA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal lpValue As String, lpcbValue As Long) As Long
Private Declare Function RegEnumKeyEx Lib "advapi32.dll" Alias "RegEnumKeyExA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpName As String, lpcbName As Long, lpReserved As Long, ByVal lpClass As String, lpcbClass As Long, lpftLastWriteTime As FILETIME) As Long
Private Declare Function RegEnumValue Lib "advapi32.dll" Alias "RegEnumValueA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpValueName As String, lpcbValueName As Long, lpReserved As Long, lpType As Long, lpData As Any, lpcbData As Long) As Long
Private Declare Function RegDeleteValue Lib "advapi32.dll" Alias "RegDeleteValueA" (ByVal hKey As Long, ByVal lpValueName As String) As Long

' Dichiarazione delle costanti usate dalle funzioni Api di Windows
Private Const KEY_ALL_ACCESS = &H3F
Private Const REG_OPTION_NON_VOLATILE = 0
Private Const REG_SZ = 1
Private Const REG_BINARY = 3
Private Const REG_DWORD = 4

' Dichiarazione dei tipi usati dalle funzioni Api di Windows
Private Type SECURITY_ATTRIBUTES
  nLength As Long
  lpSecurityDescriptor As Long
  bInheritHandle As Long
End Type

Private Type FILETIME
  dwLowDateTime As Long
  dwHighDateTime As Long
End Type

' Possibili valori della radice principale
Public Enum TRoot
  HKEY_CLASSES_ROOT = &H80000000
  HKEY_CURRENT_USER = &H80000001
  HKEY_LOCAL_MACHINE = &H80000002
  HKEY_USERS = &H80000003
  HKEY_CURRENT_CONFIG = &H80000004
  HKEY_DYN_DATA = &H80000005
End Enum

' Radice principale corrente (elemento di TRoot)
Private CurrentRoot As TRoot
' Chiave aperta in questo momento (intero per le funzioni Api)
Private CurrentKey As Long
' Chiave aperta in questo momento (stringa)
Private strCurrentKey As String

' Restituisce la stringa della chiave corrente
Public Function GetKey() As String
  GetKey = strCurrentKey
End Function

' Crea una chiave, ma non l'apre per l'uso
Public Function CreateKey(ByVal RegKey As String) As Boolean
  Dim hVal As Long
  Dim hResult As Long
  Dim Security As SECURITY_ATTRIBUTES
  CreateKey = RegCreateKeyEx(CurrentKey, RegKey, 0, vbNullString, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, Security, hVal, hResult) = 0
  RegCloseKey hVal
End Function

' Apre la chiave specificata
' Usare il parametro CanCreate per determinare anche la sua creazione
Public Function OpenKey(ByVal RegKey As String, Optional ByVal CanCreate As Boolean = False) As Boolean
  If KeyExists(RegKey) Then ' Esiste gi
    OpenKey = RegOpenKeyEx(CurrentKey, RegKey, 0, KEY_ALL_ACCESS, CurrentKey) = 0
  ElseIf CanCreate Then ' Non esiste, ma la creo
    Dim hResult As Long
    Dim Security As SECURITY_ATTRIBUTES
    OpenKey = RegCreateKeyEx(CurrentKey, RegKey, 0, vbNullString, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, Security, CurrentKey, hResult) = 0
  Else
    OpenKey = False
  End If
  strCurrentKey = IIf(OpenKey, RegKey, "")
End Function

' Chiude la chiave scrivendo i dati
Public Sub CloseKey()
  RegCloseKey CurrentKey
  CurrentKey = CurrentRoot
End Sub

' Cancella la chiave specificata
' Se RegKey="" viene cancellata la chiave corrente
Public Function DeleteKey(Optional ByVal RegKey As String = "") As Boolean
  If RegKey = "" Then ' Cancella la chiave corrente
    DeleteKey = RegDeleteKey(CurrentKey, strCurrentKey) = 0
  Else ' Cancella la chiave passata
    DeleteKey = RegDeleteKey(CurrentKey, RegKey) = 0
  End If
End Function

Public Function DeleteValue(ByVal RegValue As String) As Boolean
  DeleteValue = RegDeleteValue(CurrentKey, RegValue) = 0
End Function

' Verifica se la chiave specificata esiste o meno
' Restituisce True se esiste, False se non esiste
Public Function KeyExists(ByVal RegKey As String) As Boolean
  Dim hVal As Long
  KeyExists = RegOpenKeyEx(CurrentKey, RegKey, 0, KEY_ALL_ACCESS, hVal) = 0
  RegCloseKey hVal
End Function

' Restituisce la radice principale corrente
Public Property Get RootKey() As TRoot
  RootKey = CurrentRoot
End Property

' Imposta la radice specificata come principale
Public Property Let RootKey(ByVal Root As TRoot)
  CurrentRoot = Root
  CurrentKey = Root
End Property

Public Sub WriteInteger(ByVal Str As String, Value As Long)
  RegSetValueEx CurrentKey, Str, 0, REG_DWORD, Value, Len(Value)
End Sub

Public Function ReadInteger(ByVal Str As String) As Long
  Dim Value As Long
  RegQueryValueEx CurrentKey, Str, 0, REG_DWORD, Value, Len(Value)
  ReadInteger = Value
End Function

Public Sub WriteString(ByVal Str As String, ByVal Value As String)
  RegSetValueEx CurrentKey, Str, 0, REG_SZ, ByVal Value, Len(Value)
End Sub

Public Function ReadString(ByVal Str As String) As String
  Dim Value As String
  Value = Space(512)
  RegQueryValueEx CurrentKey, Str, 0, REG_SZ, ByVal Value, Len(Value)
  ReadString = Trim(Value)
End Function

Public Sub WriteBinary(ByVal Str As String, ByVal Value As String)
  RegSetValueEx CurrentKey, Str, 0, REG_BINARY, ByVal Value, Len(Value)
End Sub

Public Function ReadBinary(ByVal Str As String) As String
  Dim Value As String
  Value = Space(512)
  RegQueryValueEx CurrentKey, Str, 0, REG_BINARY, ByVal Value, Len(Value)
  ReadBinary = Trim(Value)
End Function

' Restituisce il valore della chiave corrente
' Si tratta della stringa in ogni chiave con nome (Predefinito)
Public Property Get KeyValue() As String
  KeyValue = ReadString(vbNullString)
End Property

' Imposta il valore della chiave corrente
' Si tratta della stringa in ogni chiave con nome (Predefinito)
Public Property Let KeyValue(ByVal Value As String)
  WriteString vbNullString, Value
End Property

' Restituisce un oggetto Collection contenente le chiavi
' presenti nella chiave attualmente aperta
Public Function EnumKeys() As Collection
  Dim Ret As New Collection
  Dim Value As String
  Dim clsName As String
  Dim Counter As Integer
  Dim keyDate As FILETIME
  Dim IsFailed As Boolean
  
  Value = Space(512)
  clsName = Space(512)
  Counter = 0
  
  Do
    IsFailed = RegEnumKeyEx(CurrentKey, Counter, ByVal Value, Len(Value), 0, clsName, Len(clsName), keyDate) <> 0
    If Not IsFailed Then
      Ret.Add Trim(Value)
      Counter = Counter + 1
    End If
  Loop Until IsFailed
  
  Set EnumKeys = Ret
End Function

' Restituisce un oggetto Collection con l'elenco di tutti i valori
' impostati nella chiave corrente
' Attenzione: non vengono fatte distinzioni dei tipi
Public Function EnumValues() As Collection
  Dim Ret As New Collection
  Dim Value As String
  Dim clsName As String
  Dim Counter As Integer
  Dim IsFailed As Boolean
  
  Value = Space(512)
  clsName = Space(512)
  Counter = 0
  
  Do
    IsFailed = RegEnumValue(CurrentKey, Counter, Value, Len(Value), 0, 0, ByVal clsName, Len(clsName))
    If Not IsFailed Then
      Ret.Add Trim(Value)
      Counter = Counter + 1
    End If
  Loop Until IsFailed
  Set EnumValues = Ret
End Function

